/*
 * Slave.cpp
 *
 * Created: 2012/4/19 2:38:14
 *  Author: Andygongyb
 */ 

#include "Slave.hpp"

const uint8_t CSlave::ADDR_BASE = 0x70;

CSlave::CSlave(uint8_t addr)
	: CTWIM_device(CTWIM_ch<CTWIM::TWIM1>::instance(),
	(addr & 0x0F) | ADDR_BASE), m_pos_thr_h(250), m_pos_thr_l(140), m_pid_set(0) {
	m_twim->set_freq(400000);		
}

void CSlave::set_servo(int16_t pos) {
	uint8_t buffer[3] = {SET_SERVO};
	memcpy(buffer + 1, &pos, 2);
	write_seq(buffer, 3);
}

void CSlave::set_servo_pro(uint8_t left, uint8_t length) {
	uint8_t buffer[3] = {SET_SERVO_PRO, left, length};
	write_seq(buffer, 3);
}

void CSlave::set_wheel(int16_t speed) {
	uint8_t buffer[3] = {SET_WHEEL};
	memcpy(buffer + 1, &speed, 2);
	write_seq(buffer, 3);
}

void CSlave::set_worm(int16_t speed) {
	uint8_t buffer[3] = {SET_WORM};
	memcpy(buffer + 1, &speed, 2);
	write_seq(buffer, 3);
}

uint16_t CSlave::get_adc(void) {
	uint8_t buffer[3] = {GET_ADC};
	read_seq(buffer, 3);
	uint16_t tmp = buffer[2] | (((uint16_t) buffer[1]) << 8);
	return tmp;
}

uint16_t CSlave::get_wheel_speed(void) {
	uint8_t buffer[3] = {GET_WHEEL_SPEED};
	read_seq(buffer, 3);
	uint16_t tmp = buffer[2] | (((uint16_t) buffer[1]) << 8);
	return tmp;
}

void CSlave::set_wheel_pid(int8_t pid_set) {
	if (pid_set > 10) {
		pid_set = 10;
	} else if (pid_set < -10) {
		pid_set = -10;
	}
	uint8_t buffer[2] = {SET_WHEEL_PID, (uint8_t) pid_set};
	write_seq(buffer, 2);
	m_pid_set = pid_set;
}

void CSlave::set_adc_thr(uint16_t l_thr, uint16_t h_thr) {
	uint8_t buffer[5] = {SET_ADC_THR};
	buffer[1] = (h_thr >> 8) & 0xFF;
	buffer[2] = h_thr & 0xFF;
	buffer[3] = (l_thr >> 8) & 0xFF;
	buffer[4] = l_thr & 0xFF;
	write_seq(buffer, 5);
	m_pos_thr_l = l_thr;
	m_pos_thr_h = h_thr;
}

void CSlave::worm_break(void) {
	uint8_t buffer = WORM_BREAK;
	write_seq(&buffer, 1);
}

void CSlave::wheel_break(void) {
	uint8_t buffer = WHEEL_BREAK;
	write_seq(&buffer, 1);
}