/*
 * SPI.hpp
 *
 *  Created on: Nov 27, 2011
 *      Author: Andygongyb
 */

#ifndef SPI_HPP_
#define SPI_HPP_

#include <avr/io.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include "Utility.hpp"

class CSPIDevice;

typedef class CSPI {
public:

	typedef enum {
		D2 = _BV(SPI2X),
		D4 = 0,
		D8 = ((_BV(SPR0)) << 1) | _BV(SPI2X),
		D16 = (_BV(SPR0)) << 1,
		D32 = ((_BV(SPR1)) << 1) | _BV(SPI2X),
		D64 = (_BV(SPR1)) << 1,
		D128 = (_BV(SPR0) | _BV(SPR1)) << 1,
	} divider_t;

	typedef enum {
		MSB = 0, LSB = _BV(DORD)
	} order_t;

	typedef enum {
		RISING = 0, FALLING = _BV(CPOL)
	} polarity_t;

	typedef enum {
		LEADING = 0, TAILING = _BV(CPHA)
	} phase_t;

	typedef struct {
		divider_t scaler;
		order_t order;
		polarity_t polarity;
		phase_t phase;
	} property_t;

	static void init(divider_t scaler = D4, order_t order = MSB,
			polarity_t polarity = RISING, phase_t phase = LEADING);
	static uint8_t trans_char(uint8_t data) {
		portENTER_CRITICAL();
		SPDR = data;
		loop_until_bit_is_set(SPSR,SPIF);
		portEXIT_CRITICAL();
		return SPDR;
	}
	static void trans_block(uint8_t *send, uint8_t count,
			uint8_t *receive = NULL) {
		for (uint8_t i = 0; i < count; i++) {
			uint8_t tmp = trans_char(*(send + i));
			if (receive)
				*(receive + i) = tmp;
		}
	}
	static void trans_block(uint8_t **buffer, uint8_t count) {
		trans_block(*buffer, count, *(buffer + 1));
	}
	static void get_property(property_t *property) {
		memcpy((void*) &m_property, (const void*) property, sizeof(property_t));
		;
	}
	static void set_property(property_t *property) {
		init(property->scaler, property->order, property->polarity,
				property->phase);
	}
private:
	CSPI(void);
	static property_t m_property;
}*pCSPI;

typedef class CSPIDevice {
private:
	volatile uint8_t *m_port;
	uint8_t m_pin;
public:
	CSPIDevice(io_set *set = NULL) {
		init(set);
	}
	void init(io_set *set = NULL);
	virtual void start(void) {
		*m_port &= ~m_pin;
	}
	virtual void stop(void) {
		*m_port |= m_pin;
	}

}*pCSPIDevice;

#endif
