/*
 * OLED_DMA.cpp
 *
 *  Created on: Apr 9, 2012
 *      Author: Andygongyb
 */

#include "OLED_DMA.hpp"

using namespace oled_names;

COLED *COLED::m_instance = NULL;

__attribute__((__section__(".oledmem"))) static uint8_t const blank[1024] = {0};

void oled_names::sel_cmd(uint8_t *last_data, uint16_t last_len) {
	gpio_clr_gpio_pin( dc);
}

void oled_names::sel_data(uint8_t *last_data, uint16_t last_len) {
	gpio_set_gpio_pin( dc);
}

void COLED::append_cmd(uint8_t* buffer, uint16_t len, bool if_need_free,
		volatile flag_t *the_flag) {
	CSPI::packet_t packet = { m_channel, buffer, len, NULL, (call_back_t)(
			&sel_cmd), NULL, the_flag };
	append(&packet, if_need_free);
}

void COLED::append_data(uint8_t* buffer, uint16_t len, bool if_need_free,
		volatile flag_t *the_flag) {
	CSPI::packet_t packet = { m_channel, buffer, len, NULL, (call_back_t)(
			&sel_data), NULL, the_flag };
	append(&packet, if_need_free);
}

COLED::COLED(void) :
	CSPIDevice(CSPI::CH1) {

	CSPIDevice::property_t property = { { { 0, 0, 0, 0, false, false, true,
			false } }, 8, 33000000 };

	// setup local bus I/O for high speed usage
	gpio_local_enable_pin_output_driver( rst);
	gpio_clr_gpio_pin(rst); // reset the OLED

	set_property(property);
	gpio_local_enable_pin_output_driver( dc);
	gpio_set_gpio_pin(dc);

	gpio_set_gpio_pin(rst);

	volatile flag_t the_flag;
	uint8_t cmd_buffer[] = { 0x8D, 0x14, 0xAF, 0xD5, 0xF0, 0xD9, 0x11, 0xDB,
			0x00, 0xA6, 0x81, 0xFF, 0xA1, 0xC8, 0x20, 0x00 /*, 0x23, 0x30*/};

	append_cmd(cmd_buffer, sizeof(cmd_buffer), false, &the_flag);
	do {
	} while (the_flag == PENDING);

	clear();

}

COLED::~COLED(void) {
	m_instance = NULL;
}

void COLED::locate(uint8_t x, uint8_t y) {
	uint8_t *buffer = new uint8_t[6];

	buffer[0] = 0x22;
	buffer[1] = y;
	buffer[2] = 7;
	buffer[3] = 0x21;
	buffer[4] = x;
	buffer[5] = 127;

	append_cmd(buffer, 6, true);
}

void COLED::put_c(char c) {
	uint8_t *buffer = new uint8_t[7];
	c -= 32;

	for (uint8_t i = 0; i < 6; i++) {
		buffer[i] = font6x8[static_cast<uint8_t> (c)][i];
	}
	buffer[6] = 0;

	append_data(buffer, 7, true);
}

void COLED::put(uint32_t data) {
	char str[12] = { 0 };
	ltoa(data, str, 10);

	*this << str;
}

CODevice& COLED::operator <<(uint32_t data) {
	put(data);
	return *this;
}

CODevice& COLED::operator <<(const char *str) {
	uint32_t len = std::strlen(str);

	uint8_t *buffer = new uint8_t[len * 7];

	for (uint8_t i = 0; i < len; i++) {
		uint8_t c = str[i] - 32;

		for (uint8_t j = 0; j < 6; j++) {
			buffer[i * 7 + j] = font6x8[static_cast<uint8_t> (c)][j];
		}
		buffer[i * 7 + 6] = 0;
	}
	append_data(buffer, len * 7, true);

	return *this;
}

void COLED::clear(void) {
	locate(0, 0);
	append_data((uint8_t*) blank, 1024);
	locate(0, 0);
}

