/*
 * SPI.cpp
 *
 *  Created on: Apr 8, 2012
 *      Author: Andygongyb
 */

#include "SPI.hpp"

CSPI *CSPI::m_instance = NULL;
volatile bool CSPI::m_if_busy = false;

CSPIDevice::property_t CSPIDevice::m_property[4] = { { { { 0 } }, 8, 1000000 },
		{ { { 0 } }, 8, 1000000 }, { { { 0 } }, 8, 1000000 }, { { { 0 } }, 8,
				1000000 } };

uint8_t CSPIDevice::m_npcs[4][2] = { { AVR32_SPI0_NPCS_0_PIN,
		AVR32_SPI0_NPCS_0_FUNCTION }, { AVR32_SPI0_NPCS_1_4_PIN,
		AVR32_SPI0_NPCS_1_4_FUNCTION }, { AVR32_SPI0_NPCS_2_3_PIN,
		AVR32_SPI0_NPCS_2_3_FUNCTION }, { AVR32_SPI0_NPCS_3_2_PIN,
		AVR32_SPI0_NPCS_3_2_FUNCTION } };

CSPI::CSPI(void) {
	bool int_sta = Is_global_interrupt_enabled();
	Disable_global_interrupt();

	m_pdca = NULL;
	m_opt_pdca = NULL;

	m_head = new CBlock();
	m_tail = m_head;
	m_len = 0;

	CInit::instance();

	gpio_enable_module_pin(AVR32_SPI0_SCK_PIN, AVR32_SPI0_SCK_FUNCTION);
	gpio_enable_module_pin(AVR32_SPI0_MISO_PIN, AVR32_SPI0_MISO_FUNCTION);
	gpio_enable_module_pin(AVR32_SPI0_MOSI_PIN, AVR32_SPI0_MOSI_FUNCTION);

	AVR32_SPI0.cr = AVR32_SPI_CR_SWRST_MASK;

	AVR32_SPI0.mr = (6 << AVR32_SPI_MR_DLYBCS_OFFSET) | (CH0
			<< AVR32_SPI_MR_PCS_OFFSET) | AVR32_SPI_MR_MODFDIS_MASK
			| AVR32_SPI_MR_MSTR_MASK; // set default channel to 0, master mode, disable the mode fault detection
	AVR32_SPI0.idr = ~0UL; // disable all interrupts; we use PDCA interrupts instead
	AVR32_SPI0.CR.spien = 1; // enable the module

	if (int_sta)
		Enable_global_interrupt();
}

CSPI::~CSPI(void) {
	detach_pdca( m_opt_pdca);
	AVR32_SPI0.CR.spidis = 1;
	CBlock *ptr1 = m_head, *ptr2 = NULL;
	CDataControl *data_control = CDataControl::instance();
	m_len = 0; // prevent potential malfunction performed by the PDCA interrupt
	while (ptr2 != m_tail) { // free the queue
		ptr2 = ptr1;
		ptr1 = static_cast<CBlock *> (ptr1->m_next);
		if (ptr2->m_packet.if_done)
			*(ptr2->m_packet.if_done) = FAILED;
		data_control->wait_for_free(ptr2);
	}
	m_if_busy = false;
	m_instance = NULL;
}

CSPI::CBlock::CBlock(packet_t* packet) {
	if (packet) {
		memcpy(&m_packet, packet, sizeof(packet_t));
		if (packet->if_done)
			*(packet->if_done) = PENDING;
	} else {
		memset(&m_packet, 0, sizeof(packet_t));
	}
}

void CSPI::start(packet_t *packet) {

	bool int_sta = Is_global_interrupt_enabled();
	Disable_global_interrupt();

	AVR32_SPI0.MR.pcs = packet->channel;
	do {

	} while (!(AVR32_SPI0.SR.tdre));

	m_pdca->set_next_mem_loc(packet->w_buffer, packet->buffer_len);
	m_opt_pdca->dis_all_int();
	m_pdca->dis_all_int();
	if (packet->r_buffer) {
		AVR32_SPI0.RDR.rd;
		m_opt_pdca->set_next_mem_loc(packet->r_buffer, packet->buffer_len);
		m_opt_pdca->en_err_int();
		m_opt_pdca->en_trans_fin_int();
		m_opt_pdca->on();
	} else {
		m_pdca->en_err_int();
		m_pdca->en_trans_fin_int();
	}
	m_pdca->on();

	if (int_sta)
		Enable_global_interrupt();
}

void CSPI::next_block(volatile flag_t if_done) {

	CBlock *tmp = CSPI::m_head;
	m_head = (CBlock *) (CSPI::m_head->m_next);

	uint8_t *last_data = m_head->m_packet.r_buffer;
	uint16_t last_len = m_head->m_packet.buffer_len;

	if (m_head->m_packet.after) {
		(*(m_head->m_packet.after))(last_data, last_len);
	}

	if (m_head->m_packet.if_done)
		*(m_head->m_packet.if_done) = if_done;
	CDataControl::instance()->wait_for_free(tmp);
	m_len--;

	if (m_len) {
		m_pdca->off();
		m_opt_pdca->off();

		if (((CBlock*) (m_head->m_next))->m_packet.before) {
			(*((CBlock*) (m_head->m_next))->m_packet.before)(last_data,
					last_len);
		}
		start(&((CBlock*) (m_head->m_next))->m_packet);
	} else {
		// no more commend, finish the transmission
		detach_pdca( m_pdca);
		detach_pdca( m_opt_pdca);
		m_if_busy = false;
	}
}

void CSPI::append(packet_t* packet, bool if_need_free) {
	bool int_sta = Is_global_interrupt_enabled();
	Disable_global_interrupt();
	m_tail->m_next = new CBlock(packet);
	m_tail = (CBlock*) (m_tail->m_next);

	if (if_need_free)
		m_tail->m_sub_buffer = static_cast<uint8_t *> (packet->w_buffer);
	m_len++;

	if (!m_if_busy) {
		m_if_busy = true;

		attach_pdca( m_pdca);
		CPDCA::property_t pdca_property = { CPDCA::BYTE, CPDCA::SPI0_TX };
		m_pdca->set_property(pdca_property);
		m_pdca->set_handler(&spi_pdca_irq);
		m_pdca->dis_all_int();
		m_pdca->get_channel()->isr;

		attach_pdca( m_opt_pdca);
		pdca_property.module = CPDCA::SPI0_RX;
		m_opt_pdca->set_property(pdca_property);
		m_opt_pdca->dis_all_int();
		m_opt_pdca->set_handler(&spi_pdca_irq);
		m_opt_pdca->get_channel()->isr;

		if (m_tail->m_packet.before) {
			(*(m_tail->m_packet.before))(NULL, 0);
		}
		start(&m_tail->m_packet);
	}
	if (int_sta)
		Enable_global_interrupt();
}

CSPIDevice::CSPIDevice(CSPI::channel_t channel, property_t *property) :
	m_channel(channel) {

	CSPI::instance();

	// translate channel_t to uint8_t
	uint8_t channel_num, channel_enum = static_cast<uint8_t> (channel);
	for (channel_num = 0; channel_enum & 0x01; channel_enum >>= 1) {
		channel_num++;
	}

	m_the_property = m_property + channel_num;

	m_csr = reinterpret_cast<volatile avr32_spi_csr_t *> (&(AVR32_SPI0.CSR0))
			+ channel_num;
	// set the csr pointer to corresponding csr

	property_t *the_property = property ? property : (m_property + channel_num);

	gpio_enable_module_pin(m_npcs[channel_num][0], m_npcs[channel_num][1]);
	set_property(*the_property);

}

CSPIDevice::~CSPIDevice(void) {
	//
}

void CSPIDevice::set_property(property_t &property) {
	std::memcpy(m_the_property, &property, sizeof(property_t));

	if ((property.data_size < 8) || (property.data_size > 16)) {
		// TODO: throw exception
	}

	int32_t baud_divisor = (CInit::instance()->get_pba_freq() + property.baud
			/ 2) / property.baud;
	if ((baud_divisor < 0) || (baud_divisor > 255)) {
		// TODO: throw exception
	}

	m_the_property->csr.st.bits_per_transfer_neglect = property.data_size - 8;
	m_the_property->csr.st.baud_rate_neglect = baud_divisor;
	m_csr->u32 = m_the_property->csr.u32;
}

void spi_pdca_irq(void) {
	CSPI *the_spi = CSPI::instance();
	volatile avr32_pdca_channel_t *the_pdca;
	if (static_cast<CSPI::CBlock *> (the_spi->m_head->m_next)->m_packet.r_buffer) {
		the_pdca = the_spi->m_opt_pdca->get_channel();
	} else {
		the_pdca = the_spi->m_pdca->get_channel();
	}

	uint32_t status = the_pdca->isr;
	if (status & AVR32_PDCA_ISR0_TERR_MASK) {
		the_spi->m_pdca->clear_err();
		the_spi->next_block(FAILED);
	} else if (status & AVR32_PDCA_ISR0_TRC_MASK) {
		the_spi->next_block(DONE);
	}

}
