/*
 * Wireless2.4GHz.cpp
 *
 *  Created on: Apr 12, 2012
 *      Author: Andygongyb
 */

#include "Wireless2.4GHz.hpp"

using namespace wireless_names;

void wireless_names::last_transfer(uint8_t *last_data, uint16_t last_len) {
	CSPI::instance()->last_transfer();
}

void wireless_names::ce_low(uint8_t *last_data, uint16_t last_len) {
	gpio_clr_gpio_pin(wireless_names::ce);
}

void wireless_names::ce_high(uint8_t *last_data, uint16_t last_len) {
	gpio_set_gpio_pin(wireless_names::ce);
}

void wireless_names::ce_high_last(uint8_t *last_data, uint16_t last_len) {
	CSPI::instance()->last_transfer();
	gpio_set_gpio_pin(wireless_names::ce);
}

CWireless *CWireless::m_instance = NULL;
CWireless::property_t CWireless::m_property = { 50, // default channel
		{ 0xE7, 0xE7, 0xE7, 0xE7, 0xC1 }, // default address
		40, // default maximum TX queue size, user defined maximum TX queue size should not exceed it
		255 // default maximum RX queue size
		};

CWireless::CWireless(void) :
	CSPIDevice(CSPI::CH0) {

	gpio_local_enable_pin_output_driver( ce);

	CSPIDevice::property_t spi_property = { { { 0, 0, 0, 0, true, false, true,
			false } }, 8, 8250000 }; // use mode 0; we also need the NPCS line stay active after transfer
	CSPIDevice::set_property(spi_property);

	ce_low(); // make sure the RF chip is in standby or power down mode

	write_reg(CONFIG, (1 << EN_CRC) | (1 << CRCO)); // power down
	write_reg(CONFIG, (1 << PWR_UP) | (1 << EN_CRC) | (1 << CRCO)); // power up in PTX mode
	cmd( FLUSH_TX); // flush both RX and TX FIFOs
	cmd( FLUSH_RX);
	write_reg(STATUS, 0x70); // clear the interrupt
	write_reg(RF_SETUP, 0x0E); // set the data rate to 250kbps with 0dBm output power
	write_reg(EN_AA, 0x01); // enable auto acknowledgment for pipe0
	write_reg(EN_RXADDR, 0x01); // enable pipe0
	write_reg(DYNPD, 0x01); // enable dynamic payload length in pipe0
	write_reg(FEATURE, (1 << EN_DPL) | (1 << EN_ACK_PAY)); // enable global dynamic payload length and payload with ACK
	write_reg(SETUP_AW, 0x03); // set address width to 5
	write_reg(SETUP_RETR, 0xFF); // wait 4ms for each retransmit, maximum 15 retransmits allowed
	// enable 2 bytes CRC

	m_tx_head = new CPackage(NULL, 0, NULL, 0);
	m_tx_tail = CWireless::m_tx_head;
	// initially the queue tail points to the queue head
	m_rx_head = new CPackage(NULL, 0, NULL, 0);
	m_rx_tail = CWireless::m_rx_head;
	m_tx_queue_len = 0;
	m_rx_queue_len = 0;
	m_failure_count = 0;

	CExtInt::property_t int_property = { CExtInt::FALLING_EDGE, true, false };
	CExtInt* ext_int = CExtInt::instance();
	ext_int->detach(CExtInt::EXTINT6);
	ext_int->attach(CExtInt::EXTINT6, &wireless_handler, &int_property);
	ext_int->en_int(CExtInt::EXTINT6);

	cpu_delay_ms(2, F_CPU); // wait for power up
	set_property(m_property);

}

CWireless::~CWireless(void) {
	ce_low(); // standby
	write_reg(CONFIG, (1 << EN_CRC) | (1 << CRCO)); // power down
	flush_rx_queue();
	m_instance = NULL;
}

uint16_t CWireless::append(uint8_t *buffer, uint16_t len,
		volatile flag_t *if_done) {
	if (len) {
		uint8_t the_len = len; // save the information for later use
		volatile flag_t *the_flag = if_done;
		if (len > 32) { // if a recursion will happen later, then redefine the data
			len = 32; // resize the data to 32 bytes
			if_done = NULL; // ignore the flag
		}
		bool int_sta = Is_global_interrupt_enabled();
		Disable_global_interrupt();
		if (m_tx_queue_len == m_property.tx_queue_size) {
			// if the number of packages in the TX queue exceeds the limit, then we should abandon one package
			// according to the FIFO rule
			if (((CPackage*) m_tx_head->m_next)->m_if_done)
				*((CPackage*) m_tx_head->m_next)->m_if_done = FAILED;
			// set the flag to 'failed'
			CWireless::m_tx_queue_len--; // decrease the counter
			CWireless::CPackage *ptr = m_tx_head; // change the m_tx_head to the next package
			CWireless::m_tx_head = (CPackage*) (ptr->m_next);
			CDataControl::instance()->wait_for_free(ptr); // will free the previous m_tx_head later
		}		

		if (!m_tx_queue_len) {
			write_payload(buffer, len); // if the the payload buffer is empty, then fill it
		}
		m_tx_tail->m_next = new CPackage(buffer, len, if_done, 0);
		// allocate the space for a new package from the heap
		m_tx_tail = (CPackage*) (m_tx_tail->m_next);
		// add the new package to the tail of the TX queue
		m_tx_queue_len++; // increase the counter
		if (the_len > 32)
			append(buffer + 32, the_len - 32, the_flag); // a recursion here to handle data block larger than 32bytes

		if (int_sta)
			Enable_global_interrupt();
	}
	return m_tx_queue_len; // return the number of data blocks currently in the queue
}

void CWireless::puts(const char *str) {
	uint16_t len = std::strlen(str);
	append((uint8_t*) str, len);
}

uint8_t CWireless::get_from_queue(uint8_t* &buffer) {
	if (m_rx_queue_len) {
		bool int_sta = Is_global_interrupt_enabled();
		Disable_global_interrupt();

		buffer = ((CPackage*) m_rx_head->m_next)->m_buffer;
		// set the pointer to the data block
		// the useful data is actually from m_rx_head->m_pDataNext
		// the m_rx_head is dummy
		uint8_t len = ((CPackage*) m_rx_head->m_next)->m_len;
		// get the length of the block
		CPackage* ptr = m_rx_head; // change the m_rx_head
		m_rx_head = (CPackage*) (ptr->m_next);
		m_rx_queue_len--; // decrease the counter
		CDataControl::instance()->wait_for_free(ptr); // will free the previous m_rx_head later

		if (int_sta)
			Enable_global_interrupt();
		return len; // return the length of the data block
	}
	buffer = NULL;
	return 0;
}

uint8_t CWireless::flush_rx_queue(void) {
	uint8_t len = m_rx_queue_len;
	CPackage *ptr1 = (CPackage*) (m_rx_head->m_next);
	// initialize the pointer to the first package
	while (len) { // use the temporary counter to ensure only those data already in the RX queue before
		// this function is called will be deleted. New data arrived during the execution of the function
		// will not be affected
		m_rx_head = (CPackage*) (ptr1->m_next); // change the head of the queue
		ptr1->free_buffer(); // will free the allocated data block later
		CPackage *ptr2 = ptr1;
		ptr1 = (CPackage*) (ptr1->m_next); // set the pointer to the next package
		CDataControl::instance()->wait_for_free(ptr2); // will free the previous m_rx_head later
		len--; // decrease the temporary counter
		m_rx_queue_len--; // decrease the global counter
	}
	return m_rx_queue_len; // return the number of package remaining in the RX queue, usually 0
}

void CWireless::set_property(property_t &property) {

	ce_low(); // to ensure being at standby mode
	write_reg(RF_CH, (property.channel) & 0x7F); // set the channel
	change_address(property.address);
	std::memcpy((void*) &m_property, (const void*) &property,
			sizeof(property_t)); // copy data
	if (property.tx_queue_size > 20)
		m_property.tx_queue_size = 20; // the TX queue size should be less than 20, or the receiver may crash

	ce_high(); // go to standby II mode
}

uint8_t CWireless::read_reg(uint8_t address, uint8_t *data) {

	*data = NOP;
	volatile flag_t the_flag = PENDING;
	uint8_t ret = cmd(R_REGISTER | (REGISTER_MASK & address), data, 1, data,
			&the_flag);
	do {
	} while (the_flag == PENDING);
	return ret; // return the value in the status register
}

uint8_t CWireless::write_reg(uint8_t address, uint8_t data) {

	volatile flag_t the_flag = PENDING;
	uint8_t ret = cmd(W_REGISTER | (REGISTER_MASK & address), &data, 1, NULL,
			&the_flag);
	do {
	} while (the_flag == PENDING);
	return ret; // return the value in the status register
}

uint8_t CWireless::cmd(uint8_t cmd, uint8_t *send, uint8_t len,
		uint8_t *receive, volatile flag_t *the_flag, call_back_t after) {

	uint8_t ret = 0;
	volatile flag_t status_flag = PENDING;

	CSPI::packet_t packet = { m_channel, &cmd, 1, &ret, NULL, NULL,
			&status_flag };
	if (len) {
		CSPIDevice::append(&packet, false); // write the command and receive the value in the status register
		packet.w_buffer = send;
		packet.buffer_len = len;
		packet.r_buffer = receive;
		packet.after = after;
		packet.if_done = the_flag;
		CSPIDevice::append(&packet, false); // write and receive a block of data with length equals to len
	} else {
		packet.after = after;
		CSPIDevice::append(&packet, false);
	}

	do {
	} while (status_flag == PENDING);
	return ret; // return the value in the status register
}

void CWireless::write_payload(uint8_t *send, uint8_t len,
		volatile flag_t *the_flag) {
	uint8_t *buffer = new uint8_t[len + 1];
	std::memcpy(static_cast<void *> (buffer + 1),
			static_cast<const void *> (send), len);
	*buffer = W_TX_PAYLOAD;

	CSPI::packet_t packet = { m_channel, buffer, len + 1, NULL, &ce_low,
			&ce_high_last, the_flag };
	CSPIDevice::append(&packet, true);
}

void wireless_handler(uint8_t int_num) {
	static CWireless *the_wireless = CWireless::instance();
	uint8_t status = the_wireless->cmd(NOP); // read the status register
	the_wireless->write_reg(CWireless::STATUS, 0x7E); // clear the interrupt bits
	ce_low(); // go to standby mode
	if (status & (1 << RX_DR)) { // new data available in the RX FIFO
		Disable_global_interrupt();
		if (the_wireless->m_rx_queue_len
				== the_wireless->m_property.rx_queue_size) {
			// if the number of packages in the RX queue exceeds the limit, then we should abandon one package
			// according to the FIFO rule
			the_wireless->m_rx_queue_len--; // decrease the counter
			CWireless::CPackage *ptr = the_wireless->m_rx_head; // change the m_rx_head to the next package
			the_wireless->m_rx_head = (CWireless::CPackage*) (ptr->m_next);
			ptr->free_buffer(); // free the data block in that package
			CDataControl::instance()->wait_for_free(ptr); // will free the previous m_rx_head later
		}
		uint8_t pipe_id = (status & RX_P_NO) >> 1, len = NOP; // put the pipe ID of the latest package in pipe_id
		Enable_global_interrupt();
		volatile flag_t flag;
		the_wireless->cmd(R_RX_PL_WID, &len, 1, &len, &flag); // read the length of the package
		do {} while (flag == PENDING);
		the_wireless->m_rx_tail->m_next = new CWireless::CPackage(NULL, len,
				NULL, pipe_id);
		// allocate the space for a new package from the heap
		the_wireless->m_rx_tail
				= (CWireless::CPackage*) (the_wireless->m_rx_tail->m_next);
		// add the new package to the tail of the RX queue
		the_wireless->m_rx_tail->m_buffer = new uint8_t[len];
		// allocate the space for the received data
		the_wireless->cmd(R_RX_PAYLOAD, the_wireless->m_rx_tail->m_buffer, len,
				the_wireless->m_rx_tail->m_buffer, NULL, &ce_high_last); // read the received data from the RX FIFO
		the_wireless->m_rx_queue_len++; // increase the counter
	}
	if (status & (1 << TX_DS)) { // a transmission is successful
		Disable_global_interrupt();
		if (((CWireless::CPackage*) the_wireless->m_tx_head->m_next)->m_if_done)
			*((CWireless::CPackage*) the_wireless->m_tx_head->m_next)->m_if_done
					= DONE; // set the flag
		the_wireless->m_tx_queue_len--; // decrease the counter
		CWireless::CPackage *ptr = the_wireless->m_tx_head; // change the m_tx_head to the next package
		the_wireless->m_tx_head = (CWireless::CPackage*) (ptr->m_next);
		CDataControl::instance()->wait_for_free(ptr); // will free the previous m_tx_head later
		Enable_global_interrupt();
		if (the_wireless->m_tx_queue_len) // if more packages are in the TX queue
			the_wireless->cmd(
					W_TX_PAYLOAD,
					((CWireless::CPackage*) the_wireless->m_tx_head->m_next)->m_buffer,
					((CWireless::CPackage*) the_wireless->m_tx_head->m_next)->m_len,
					NULL, NULL, &ce_high_last);
		// write a new package to the TX FIFO
	}
	if (status & (1 << MAX_RT)) { // a transmission reaches the maximum re-transmission count
		the_wireless->cmd(FLUSH_TX); // flush the TX FIFO
		the_wireless->cmd(
				W_TX_PAYLOAD,
				((CWireless::CPackage*) the_wireless->m_tx_head->m_next)->m_buffer,
				((CWireless::CPackage*) the_wireless->m_tx_head->m_next)->m_len,
				NULL, NULL, &ce_high_last);
		// rewrite the package to the TX FIFO
		if (the_wireless->m_failure_count + 1) // increase the failure counter
			the_wireless->m_failure_count++;
	}
}

