/*
 * Wireless2.4GHz.hpp
 *
 *  Created on: Mar 4, 2011
 *      Author: Andygongyb
 */

#ifndef WIRELESS_HPP_
#define WIRELESS_HPP_

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <inttypes.h>
#include <alloca.h>
#include <util/delay.h>
#include "SPI.hpp"
#include "Utility.hpp"

namespace wireless_names {

const uint8_t MASK_RX_DR = 6;
const uint8_t MASK_TX_DS = 5;
const uint8_t MASK_MAX_RT = 4;
const uint8_t EN_CRC = 3;
const uint8_t CRCO = 2;
const uint8_t PWR_UP = 1;
const uint8_t PRIM_RX = 0;
const uint8_t ENAA_P5 = 5;
const uint8_t ENAA_P4 = 4;
const uint8_t ENAA_P3 = 3;
const uint8_t ENAA_P2 = 2;
const uint8_t ENAA_P1 = 1;
const uint8_t ENAA_P0 = 0;
const uint8_t ERX_P5 = 5;
const uint8_t ERX_P4 = 4;
const uint8_t ERX_P3 = 3;
const uint8_t ERX_P2 = 2;
const uint8_t ERX_P1 = 1;
const uint8_t ERX_P0 = 0;
const uint8_t AW = 0;
const uint8_t ARD = 4;
const uint8_t ARC = 0;
const uint8_t CONT_WAVE = 7;
const uint8_t RF_DR_LOW = 5;
const uint8_t PLL_LOCK = 4;
const uint8_t RF_DR_HIGH = 3;
const uint8_t RF_PWR1 = 2;
const uint8_t RF_PWR0 = 1;
const uint8_t RX_DR = 6;
const uint8_t TX_DS = 5;
const uint8_t MAX_RT = 4;
const uint8_t TX_REUSE = 6;
const uint8_t TX_FULL = 5;
const uint8_t TX_EMPTY = 4;
const uint8_t RX_FULL = 1;
const uint8_t RX_EMPTY = 0;
const uint8_t EN_DPL = 2;
const uint8_t EN_ACK_PAY = 1;
const uint8_t EN_DYN_ACK = 0;

const uint8_t PLOS_CNT_MASK = 0xF0;
const uint8_t ARC_CNT_MASK = 0x0F;

const uint8_t R_REGISTER = 0x00;
const uint8_t W_REGISTER = 0x20;
const uint8_t REGISTER_MASK = 0x1F;
const uint8_t RX_P_NO = 0x0E;
const uint8_t R_RX_PAYLOAD = 0x61;
const uint8_t W_TX_PAYLOAD = 0xA0;
const uint8_t R_RX_PL_WID = 0x60;
const uint8_t W_ACK_PAYLOAD = 0xA8;
const uint8_t FLUSH_TX = 0xE1;
const uint8_t FLUSH_RX = 0xE2;
const uint8_t REUSE_TX_PL = 0xE3;
const uint8_t NOP = 0xFF;

}

extern "C" void INT1_vect(void)__attribute__((signal,used));

typedef class CWireless: private CSPIDevice {
public:

	typedef enum {
		CONFIG = 0x00,
		EN_AA = 0x01,
		EN_RXADDR = 0x02,
		SETUP_AW = 0x03,
		SETUP_RETR = 0x04,
		RF_CH = 0x05,
		RF_SETUP = 0x06,
		STATUS = 0x07,
		OBSERVE_TX = 0x08,
		PRD = 0x09,
		RX_ADDR_P0 = 0x0A,
		RX_ADDR_P1 = 0x0B,
		RX_ADDR_P2 = 0x0C,
		RX_ADDR_P3 = 0x0D,
		RX_ADDR_P4 = 0x0E,
		RX_ADDR_P5 = 0x0F,
		TX_ADDR = 0x10,
		RX_PW_P0 = 0x11,
		RX_PW_P1 = 0x12,
		RX_PW_P2 = 0x13,
		RX_PW_P3 = 0x14,
		RX_PW_P4 = 0x15,
		RX_PW_P5 = 0x16,
		FIFO_STATUS = 0x17,
		DYNPD = 0x1C,
		FEATURE = 0x1D,
	} reg_map_t;

	typedef struct {
		uint8_t channel;
		uint8_t rx_p0_addr[5];
		uint8_t rx_addr_domain[4];
		uint8_t rx_addr_lsb[5];
		uint8_t max_tx_len;
		uint8_t max_rx_len;
	} property_t;

private:
	typedef class CData {
	private:
		CData *m_next;
		uint8_t m_len;
		uint8_t m_pipe_id;
		uint8_t m_buffer[32];
		bool *m_if_done;

	public:

		CData(void) {
			m_next = NULL;
		}

		void init(uint8_t len, bool *if_done, uint8_t pipe_id) {
			m_next = NULL;
			m_len = len;
			m_pipe_id = pipe_id;
			m_if_done = if_done;
		}

		friend class CWireless;
		friend void INT1_vect(void);
	}*pCData;
	
	CData m_data_bank[23];

	volatile uint8_t *m_port_ce;
	uint8_t m_pin_ce;
	static CWireless *m_instance;

	property_t m_property;

	CData *m_tx_head;
	CData *m_tx_tail;
	volatile uint8_t m_tx_len;

	CData *m_rx_head;
	CData *m_rx_tail;
	volatile uint8_t m_rx_len;

	volatile uint8_t m_failure_count;

	CData *m_free_block;
	volatile uint8_t m_free_count;

	CWireless(void);

	void ce_down(void) {
		*m_port_ce &= ~m_pin_ce;
	}
	void ce_up(void) {
		*m_port_ce |= m_pin_ce;
	}
	uint8_t cmd(uint8_t cmd, uint8_t *send = NULL, uint8_t len = 0,
			uint8_t *receive = NULL);

	void add_to_free_list(CData *ptr) {
		ptr->m_next = m_free_block;
		m_free_block = ptr;
		m_free_count++;
	}

	CData *get_from_free_list(void) {

		if (m_free_count) {
			CData *ptr = m_free_block;
			m_free_block = m_free_block->m_next;
			m_free_count--;

			return ptr;
		}

		return NULL;
	}

public:
	~CWireless(void);

	static CWireless *instance(void) {
		if (!m_instance) {
			m_instance = new CWireless();
		}
		return m_instance;
	}

	void set_property(property_t *property);
	void get_property(property_t *property) {
		memcpy((void*) property, (const void*) &m_property, sizeof(property_t));
	}

	uint8_t append(uint8_t *buffer, uint8_t len, bool *if_done = NULL,
			uint8_t pipe_id = 0);
	uint8_t get(uint8_t *buffer);

	uint8_t next_len(void);

	uint8_t read_reg(uint8_t addr, uint8_t *data);
	uint8_t read_reg(uint8_t addr, uint8_t *buffer, uint8_t len) {
		for (uint8_t i = 0; i < len; i++)
			*(buffer + i) = wireless_names::NOP; // set the data in the buffer to NOP to prevent malfunction
		return cmd(
				wireless_names::R_REGISTER | (wireless_names::REGISTER_MASK
						& addr), buffer, len, buffer);
	}
	uint8_t write_reg(uint8_t addr, uint8_t data);
	uint8_t write_reg(uint8_t addr, uint8_t *buffer, uint8_t len) {
		return cmd(
				wireless_names::W_REGISTER | (wireless_names::REGISTER_MASK
						& addr), buffer, len);
	}

	uint8_t flush_rx(void) {
		return cmd(wireless_names::FLUSH_RX);
	}

	uint8_t get_tx_len(void) {
		return m_tx_len;
	}
	uint8_t get_rx_len(void) {
		return m_rx_len;
	}

	friend void INT1_vect(void);
}*pCWireless;

#endif
